package java.net;

import java.util.Comparator;
import sun.util.logging.PlatformLogger;
import java.io.IOException;
import java.util.Iterator;
import java.util.Collections;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class CookieManager extends CookieHandler
{
    private CookiePolicy policyCallback;
    private CookieStore cookieJar;
    
    public CookieManager() {
        this(null, null);
    }
    
    public CookieManager(CookieStore cookieJar, CookiePolicy cookiePolicy) {
        this.cookieJar = null;
        this.policyCallback = ((cookiePolicy == null) ? CookiePolicy.ACCEPT_ORIGINAL_SERVER : cookiePolicy);
        if (cookieJar == null) {
            this.cookieJar = new InMemoryCookieStore();
        }
        else {
            this.cookieJar = cookieJar;
        }
    }
    
    public void setCookiePolicy(CookiePolicy policyCallback) {
        if (policyCallback != null) {
            this.policyCallback = policyCallback;
        }
    }
    
    public CookieStore getCookieStore() {
        return this.cookieJar;
    }
    
    @Override
    public Map<String, List<String>> get(URI uri, Map<String, List<String>> map) throws IOException {
        if (uri == null || map == null) {
            throw new IllegalArgumentException("Argument is null");
        }
        HashMap<String, List<String>> hashMap = new HashMap();
        if (this.cookieJar == null) {
            return Collections.unmodifiableMap(hashMap);
        }
        int equalsIgnoreCase = "https".equalsIgnoreCase(uri.getScheme());
        ArrayList<HttpCookie> list = new ArrayList();
        String path = uri.getPath();
        if (path == null || path.isEmpty()) {
            path = "/";
        }
        for (HttpCookie httpCookie : this.cookieJar.get(uri)) {
            if (this.pathMatches(path, httpCookie.getPath()) && (equalsIgnoreCase || !httpCookie.getSecure())) {
                if (httpCookie.isHttpOnly()) {
                    String scheme = uri.getScheme();
                    if (!"http".equalsIgnoreCase(scheme) && !"https".equalsIgnoreCase(scheme)) {
                        continue;
                    }
                }
                String portlist = httpCookie.getPortlist();
                if (portlist != null && !portlist.isEmpty()) {
                    int port = uri.getPort();
                    if (port == -1) {
                        port = ("https".equals(uri.getScheme()) ? 443 : 80);
                    }
                    if (isInPortList(portlist, port)) {
                        list.add(httpCookie);
                    }
                    else {
                        continue;
                    }
                }
                else {
                    list.add(httpCookie);
                }
            }
            else {
                continue;
            }
        }
        hashMap.put("Cookie", this.sortByPath(list));
        return Collections.unmodifiableMap(hashMap);
    }
    
    @Override
    public void put(URI obj, Map<String, List<String>> map) throws IOException {
        if (obj == null || map == null) {
            throw new IllegalArgumentException("Argument is null");
        }
        if (this.cookieJar == null) {
            return;
        }
        PlatformLogger logger = PlatformLogger.getLogger("java.net.CookieManager");
        for (String s : map.keySet()) {
            if (s == null) {
                continue;
            }
            if (!s.equalsIgnoreCase("Set-Cookie2") && !s.equalsIgnoreCase("Set-Cookie")) {
                continue;
            }
            for (String str : (List)map.get(s)) {
                try {
                    List<Object> list;
                    try {
                        list = HttpCookie.parse(str);
                    }
                    catch (IllegalArgumentException ex) {
                        list = Collections.emptyList();
                        if (logger.isLoggable(PlatformLogger.Level.SEVERE)) {
                            logger.severe("Invalid cookie for " + obj + ": " + str);
                        }
                    }
                    for (HttpCookie httpCookie : list) {
                        if (httpCookie.getPath() == null) {
                            String path = obj.getPath();
                            if (!path.endsWith("/")) {
                                int lastIndex = path.lastIndexOf("/");
                                if (lastIndex > 0) {
                                    path = path.substring(0, lastIndex + 1);
                                }
                                else {
                                    path = "/";
                                }
                            }
                            httpCookie.setPath(path);
                        }
                        if (httpCookie.getDomain() == null) {
                            String s2 = obj.getHost();
                            if (s2 != null && !s2.contains(".")) {
                                s2 += ".local";
                            }
                            httpCookie.setDomain(s2);
                        }
                        String portlist = httpCookie.getPortlist();
                        if (portlist != null) {
                            int port = obj.getPort();
                            if (port == -1) {
                                port = ("https".equals(obj.getScheme()) ? 443 : 80);
                            }
                            if (portlist.isEmpty()) {
                                httpCookie.setPortlist("" + port);
                                if (this.shouldAcceptInternal(obj, httpCookie)) {
                                    this.cookieJar.add(obj, httpCookie);
                                }
                                else {
                                    continue;
                                }
                            }
                            else if (isInPortList(portlist, port) && this.shouldAcceptInternal(obj, httpCookie)) {
                                this.cookieJar.add(obj, httpCookie);
                            }
                            else {
                                continue;
                            }
                        }
                        else if (this.shouldAcceptInternal(obj, httpCookie)) {
                            this.cookieJar.add(obj, httpCookie);
                        }
                        else {
                            continue;
                        }
                    }
                }
                catch (IllegalArgumentException ex2) {}
            }
        }
        return;
    }
    
    private boolean shouldAcceptInternal(URI uri, HttpCookie httpCookie) {
        try {
            return this.policyCallback.shouldAccept(uri, httpCookie);
        }
        catch (Exception ex) {
            return false;
        }
    }
    
    private static boolean isInPortList(String substring, int n) {
        for (int i = substring.indexOf(","); i > 0; substring = substring.substring(i + 1), i = substring.indexOf(",")) {
            try {
                if (Integer.parseInt(substring.substring(0, i)) == n) {
                    return true;
                }
            }
            catch (NumberFormatException ex) {}
        }
        if (!substring.isEmpty()) {
            try {
                if (Integer.parseInt(substring) == n) {
                    return true;
                }
            }
            catch (NumberFormatException ex2) {}
        }
        return false;
    }
    
    private boolean pathMatches(String s, String prefix) {
        return s == prefix || (s != null && prefix != null && s.startsWith(prefix));
    }
    
    private List<String> sortByPath(List<HttpCookie> list) {
        Collections.sort(list, new CookieManager.CookiePathComparator());
        ArrayList<String> list2 = new ArrayList();
        for (HttpCookie httpCookie : list) {
            if (list.indexOf(httpCookie) == 0 && httpCookie.getVersion() > 0) {
                list2.add("$Version=\"1\"");
            }
            list2.add(httpCookie.toString());
        }
        return list2;
    }
    
    static class CookiePathComparator implements Comparator<HttpCookie>
    {
        @Override
        public int compare(HttpCookie httpCookie, HttpCookie httpCookie2) {
            if (httpCookie == httpCookie2) {
                return 0;
            }
            if (httpCookie == null) {
                return -1;
            }
            if (httpCookie2 == null) {
                return 1;
            }
            if (!httpCookie.getName().equals(httpCookie2.getName())) {
                return 0;
            }
            if (httpCookie.getPath().startsWith(httpCookie2.getPath())) {
                return -1;
            }
            if (httpCookie2.getPath().startsWith(httpCookie.getPath())) {
                return 1;
            }
            return 0;
        }
    }
}
